'２次元捩り解析V67の標準ﾓｼﾞｭｰﾙ        ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module Tors2DCom
    '一様断面軸の弾性捩り解析
    '節点の剪断応力を算出
    '各種の断面定数も出力
    'ｱｲｿﾊﾟﾗﾒﾄﾘｯｸ四角形要素使用
    '入力ﾃﾞｰﾀは別途「ＦＥＭ断面データ」ｿﾌﾄにより作成

    'V56での改良点・・・各画面でボタンを読みやすい色に改善
    'V57での改良点・・・Visual Basic 2017に変更
    'V58での改良点・・・解析データで要素の削除を追加
    'V59での改良点・・・解析計算開始前に入力データのエラーチェックを追加
    'V60での改良点・・・プログラムファイルの文字コードをUTF-8に統一、
    '                   データファイルの文字コードを標準でUTF-8に変更
    '                   （従来のShift-JISもオプションで可能）
    'V61での改良点・・・ファイル入出力画面でキャンセル処理の改善
    'V62での改良点・・・データ表画面で表計算ソフトから貼り付け時に次行に0が入るのを防止
    'V63での改良点・・・入力データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V64での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                   （これまでは最長方向の座標軸を採用）
    'V65での改良点・・・サブメニューの中間節点自動計算を廃止
    'V66での改良点・・・解析データのデータチェックに要素共通辺の節点を追加
    'V67での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$                 '計算結果表示データ
    Public Cr$                      'CR+LFのコード
    Public DirOpen$                 '立ち上げ時のディレクトリー名
    Public Fadd$                    '連結するファイル名
    Public Fm$                      '入力データのファイル名
    Public Fy$                      '計算結果のファイル名
    Public Title$                   'タイトル

    Public Timst As Date            '計算開始時刻

    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力）
    Public Kaxis As Integer         '作図のＸＹ軸の入換え（0:なし、1:有り）
    Public Kcolorfill As Integer    '等高線図のカラーフィルの有無（0:なし、1:有り）
    Public KcomSide As Integer      '要素辺の表示有無（0:なし、1:有り）
    Public KcontourPoint As Integer '等高線図で場所･数値表示の有無（0:なし、1:有り）
    Public KdataBackup As Integer   '入力データのバックアップファイル作成（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindPost As Integer      '後処理の種類（1:等高線図、2:端数処理、主要値表示）
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KnewUFileB As Integer    '最近使用ﾌｧｲﾙBの更新有無
    Public KnumDisp As Integer      '構造作画での番号表示（0:なし、1:有り）
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public Kvalue As Integer        '作画の変数ｲﾝﾃﾞｯｸｽ
    Public Nall As Integer          '変数の総数
    Public Ndim As Integer          '１節点当りの変数の数(1)
    Public NDisp As Integer         '等高線の分割数
    Public Ndel As Integer          '削除要素数
    Public Nelem As Integer         '要素数
    Public Ngaus As Integer         'ガウス積分点数(3)
    Public Npoint As Integer        '節点数
    Public Nvar As Integer          '１要素当りの変数の総数

    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public Gshear As Single         '剪断弾性係数
    Public TMoment As Single        'モーメント荷重
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public VdispMax As Single       '作画変数の最高値
    Public VdispMin As Single       '作画変数の最低値
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public Yupper As Single         '作画領域のY最大値

    Public UFileB(6) As String      '最近使用した解析ﾌｧｲﾙの保存配列

    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdel() As Integer        '削除要素番号
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号

    Public Ds(8), Dt(8) As Single   '形状関数のs偏微分、t偏微分
    Public Dx(8), Dy(8) As Single   '形状関数のx偏微分、y偏微分
    Public PosGaus(3) As Single     'ｶﾞｳｽ積分点の位置(於要素座標）
    Public Tf() As Single           'FEM計算の右辺ベクトル
    Public WeightG(3) As Single     'ｶﾞｳｽ積分点の重み
    Public X(), Y() As Single       '節点座標
    Public Xdraw(5), Ydraw(5) As Single     '色塗り多角形の座標
    Public XE(8), YE(8) As Single   '要素計算の配列
    Public Xmouse(), Ymouse() As Single     'ﾏｳｽ位置座標

    Sub AxisCheck2D(ByRef icheck As Integer, ByRef k1 As Integer, _
        ByRef k2 As Integer, ByRef ratio As Single, ByRef xmin As Single, _
        ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'k1,k2･･･要素の開始、終了番号
        'ratio･･･作画直角回転なしの縦横比上限（指定値）
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim i, ie, ik, j As Integer
        Dim b1, b2, ratiob As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = X(i) : xmax = xmin
                        ymin = Y(i) : ymax = ymin
                    Else
                        If X(i) < xmin Then
                            xmin = X(i)
                        ElseIf X(i) > xmax Then
                            xmax = X(i)
                        End If
                        If Y(i) < ymin Then
                            ymin = Y(i)
                        ElseIf Y(i) > ymax Then
                            ymax = Y(i)
                        End If
                    End If
                Next j
            End If
        Next ie
        b1 = xmax - xmin
        b2 = ymax - ymin
        If b1 = 0 Then
            icheck = 0
        Else
            ratiob = b2 / b1
            If ratiob > ratio Then
                icheck = 1
            Else
                icheck = 0
            End If
        End If

    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf()･･･節点の座標
        Dim i, ie, kind, nb As Integer
        Dim b1, b2, pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        ratio = 1.2
        Kaxis = 0
        Call AxisCheck2D(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        b1 = pa - XaxisMin
        b2 = qa - YaxisMin
        If b2 > b1 Then
            b1 = b2
        End If
        b2 = b1 * 0.00002
        Call RoundPrep(kind, nb, b2)    '丸め準備

        For i = 1 To Npoint
            xbuf(i) = X(i)
            ybuf(i) = Y(i)
        Next i
        For i = 1 To Npoint
            If xbuf(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If ybuf(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub BandWidth2(ByRef mb As Integer, ByVal kcbuf() As Integer)
        'ﾊﾞﾝﾄﾞ幅の計算
        'kcbuf()･･･元の節点の変数順序
        'mb･･･ﾊｰﾌﾊﾞﾝﾄﾞ幅
        Dim i, ie, ii, j, jj, k, mx As Integer

        mx = 0
        For ie = 1 To Nelem
            For ii = 1 To 7
                i = NNelem(ie, ii)
                For jj = ii + 1 To 8
                    j = NNelem(ie, jj)
                    k = Math.Abs(kcbuf(i) - kcbuf(j))
                    If k > mx Then mx = k
                Next jj
            Next ii
        Next ie
        mb = Ndim * (mx + 1)

    End Sub

    Sub DataCheck(ByVal kdisp As Integer, ByRef num As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kdisp･･･エラー無しのメッセージの有無（0:なし、1:有り）
        'num･･･エラーの数（上限10）
        Dim aa$, aerr1$
        Dim ie, ip, j, k, nb, nx As Integer
        Dim b1, b2 As Single
        Dim abuf$(10)
        aerr1$ = "になっています！"
        num = 0

        If Npoint < 8 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  節点数が不足です！"
        End If
        If Nelem < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  要素数が" + Str$(Nelem) + aerr1$
        End If
        If Gshear = 0 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  剪断弾性係数が入力されていません！"
        End If
        If TMoment = 0 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  捩りモーメントが入力されていません！"
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            For j = 1 To 8
                nb = NNelem(ie, j)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  要素番号" + Str$(ie) + "の"
                    abuf$(num) = aa$ + " 節点番号" + Str$(nb) + "は定義されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                If j < 8 Then
                    For k = j + 1 To 8
                        nx = NNelem(ie, k)
                        If nb = nx Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                            abuf$(num) = aa$ + " ２つの節点番号が" + Str$(nb) + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    Next k
                End If
            Next j
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Npoint - 1
            For j = ip + 1 To Npoint
                b1 = X(ip) - X(j)
                b2 = Y(ip) - Y(j)
                If b1 = 0 And b2 = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  節点番号" + Str$(ip) + "、" + Str$(j) + "が"
                    abuf$(num) = aa$ + " 同じ座標位置" + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ip

        '不要節点のﾁｪｯｸ
        Dim kbuf(Npoint) As Integer
        For ie = 1 To Nelem
            For j = 1 To 8
                kbuf(NNelem(ie, j)) = 1
            Next j
        Next ie
        For ip = 1 To Npoint
            If kbuf(ip) = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が使われていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        Next ip

        '要素共通辺のﾁｪｯｸ
        ReDim kbuf(Npoint)
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) = 0 Then
                    kbuf(ip) = 1    '頂点節点に指定
                End If
            Next j
        Next ie
        For ie = 1 To Nelem
            For j = 2 To 8 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ie

        If num = 0 Then
            If kdisp = 1 Then
                aa$ = "データのエラーチェックが終了しました！"
                MsgBox(aa$, 0, "データＯＫ")
            End If
        Else
            Call ErrorDisp(0, num, abuf$) 'ｴﾗｰの表示
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        'ii･･･ﾌｧｲﾙ番号
        Dim aa$
        Dim i, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "Torsion2D" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Gshear = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        TMoment = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ngaus = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ndim = dat(1)

        ReDim NNelem(Nelem, 8)
        ReDim X(Npoint), Y(Npoint)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To 8
                NNelem(i, j) = dat(j)
            Next j
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            X(i) = dat(1)
            Y(i) = dat(2)
        Next i

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "プログラム名" + "," + "Torsion2D"
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        aa$ = "剪断弾性係数" + "," + Str(Gshear)
        sw.WriteLine(aa$)
        aa$ = "捩りモーメント" + "," + Str(TMoment)
        sw.WriteLine(aa$)
        aa$ = "ガウス積分点数" + "," + Str(Ngaus)
        sw.WriteLine(aa$)
        aa$ = "１節点当り変数の数" + "," + Str(Ndim)
        sw.WriteLine(aa$)

        sw.WriteLine("要素データ (要素番号、8節点番号)")
        For i = 1 To Nelem
            aa$ = Str$(i)
            For j = 1 To 8
                aa$ = aa$ + ",   " + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        sw.WriteLine("節点データ（節点番号、Ｘ座標、Ｙ座標）")
        For i = 1 To Npoint
            aa$ = Str$(i) + ",   " + Str$(X(i)) + ",   " + Str$(Y(i))
            sw.WriteLine(aa$)
        Next i

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawNodeVar(ByRef gr As Graphics, ByVal aa As String, _
    ByVal xb As Single, ByVal yb As Single)
        '節点変数の描画
        'aa･･･変数のテキスト
        'xb,yb･･･描画位置
        Dim xbw, ybw As Single
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", 10, FontStyle.Bold)

        xbw = xb - 1
        ybw = yb
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb
        ybw = yb - 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        ybw = yb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        gr.DrawString(aa$, fnt2, Brushes.Purple, xb, yb)

    End Sub

    Sub DrawScale(ByRef icheck As Integer, ByVal dpixb As Single, _
        ByVal dpiyb As Single, ByVal xlen As Single, ByVal ylen As Single, _
        ByVal xmin As Single, ByVal xmax As Single, _
        ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb
        b1 = xmax - xmin : b2 = ymax - ymin
        Select Case icheck
            Case 0
                If Cfix < 1 Then
                    If b1 <> 0 Then
                        cx = xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2 / Cfix
                    End If
                Else
                    If b1 <> 0 Then
                        cx = Cfix * xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2
                    End If
                End If
                If b1 = 0 Then
                    XYscale = cy
                ElseIf b2 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
            Case 1
                If Cfix < 1 Then
                    If b2 <> 0 Then
                        cx = xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1 / Cfix
                    End If
                Else
                    If b2 <> 0 Then
                        cx = Cfix * xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1
                    End If
                End If
                If b2 = 0 Then
                    XYscale = cy
                ElseIf b1 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
        End Select

    End Sub

    Sub Elem2DPrep()
        '構造図作画準備
        Dim ie As Integer
        Dim pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        KE1 = 1
        KE2 = Nelem
        KnumDisp = 0
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        KnumDisp = 0

        ratio = 1.2
        Kaxis = 0
        Call AxisCheck2D(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

    End Sub

    Sub ElemSearch2(ByRef k1 As Integer, ByRef k2 As Integer)
        '拡大作画する要素の探索
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        Dim ib, ic, ie, j, kmax, kmin As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    b1 = X(ib) - Xmouse(1)
                    b2 = X(ib) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        If Kaxis = 0 Then
                            b1 = Y(ib) - Ymouse(2)
                            b2 = Y(ib) - Ymouse(1)
                        Else
                            b1 = Y(ib) - Ymouse(1)
                            b2 = Y(ib) - Ymouse(2)
                        End If
                        If b1 >= 0 And b2 <= 0 Then
                            If j = 8 Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If
    End Sub

    Private Sub ErrorDisp(ByVal kind As Integer, ByVal num As Integer,
        ByRef abuf() As String)
        'ｴﾗｰの表示
        'kind･･･打切り表示の有無（0:なし、1:有り）
        'num･･･ｴﾗｰの個数
        'abuf$()･･･ｴﾗｰの表示文
        Dim aa$, nc$
        Dim i As Integer
        nc$ = Chr(13) + Chr(10)
        aa$ = ""
        For i = 1 To num
            aa = aa$ + abuf(i) + nc$
        Next i
        If kind > 0 Then
            aa$ = aa$ + "データチェックを打切りました！！"
        End If
        MsgBox(aa$, 0, "データエラー")

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub MiniSort2(ByVal kind As Integer, ByRef nsort() As Integer, _
    ByVal work() As Single, ByVal xbuf() As Single, ByVal ybuf() As Single)
        'ﾐﾆｿｰﾄ2
        '同じ変数値の場合に、第2座標値でソート
        'kind･･･主ｿｰﾄの座標軸番号（1:X軸、2:Y軸）
        'nsort()･･･元の変数番号のｿｰﾄ順番号
        'work()･･･ｿｰﾄ変数値
        'xbuf(),ybuf()･･･節点の座標
        Dim i, j, jb, jx, nb, n1, n2, nx As Integer
        Dim b1 As Single
        Dim nsort2(10) As Integer
        Dim work2(10) As Single

        i = 1
        Do
            If work(i) = work(i + 1) Then
                b1 = work(i)
                n1 = i
                For j = i + 1 To Npoint
                    If b1 = work(j) Then
                        n2 = j
                    Else
                        Exit For
                    End If
                Next j
                nx = n2 - n1 + 1
                ReDim nsort2(nx), work2(nx)
                jb = 0
                For j = n1 To n2
                    jb = jb + 1
                    nsort2(jb) = jb
                    nb = nsort(j)
                    If kind = 1 Then
                        work2(jb) = ybuf(nb)
                    Else
                        work2(jb) = xbuf(nb)
                    End If
                Next j
                Call ShellSortStd(nx, nsort2, work2)    'ｼｪｯﾙｿｰﾄ基本
                For j = 1 To nx
                    jb = n1 + nsort2(j) - 1
                    nsort2(j) = nsort(jb)
                Next j
                For j = 1 To nx
                    jx = n1 + j - 1
                    nsort(jx) = nsort2(j)
                Next j
                i = n2 + 1
            Else
                i = i + 1
            End If
        Loop While i < Npoint

    End Sub

    Sub MouseCalc(ByVal xa As Single, ByRef xb As Single, ByVal ya As Single, ByRef yb As Single)
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        'xa,ya･･･ﾏｳｽ点座標
        'xb,yb･･･ﾓﾃﾞﾙ座標
        If Cfix < 1 Then
            If Kaxis = 0 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                yb = (xa - Xlower) / XYscale + YaxisMin
            End If
        Else
            If Kaxis = 0 Then
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            Else
                xb = (ya - Ylower) / XYscale + XaxisMin
                yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
            End If
        End If

    End Sub

    Sub MouseCalc2()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                If Kaxis = 0 Then
                    xb = (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                    yb = (xa - Xlower) / XYscale + YaxisMin
                End If
            Else
                If Kaxis = 0 Then
                    xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale + XaxisMin
                    yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
                End If
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MousePointSearch(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single)
        'ﾏｳｽ点の節点の探索
        'k1,k2･･･要素の開始、終了番号
        'ky･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点のﾓﾃﾞﾙ座標位置
        Dim ib, ie, j As Integer
        Dim b1, b2, b3, rlen2 As Single
        Dim kbuf(Npoint) As Integer

        kp = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        kbuf(ib) = 1
                        b1 = X(ib) - xb
                        b2 = Y(ib) - yb
                        b3 = b1 * b1 + b2 * b2
                        If kp = 0 Or b3 < rlen2 Then
                            kp = ib
                            rlen2 = b3
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub NumCheck(ByRef ic As Integer, ByRef i1 As Integer, _
        ByRef i2 As Integer, ByRef j1 As Integer, _
        ByRef j2 As Integer)
        '要素辺の重なりのﾁｪｯｸ
        'ic･･･要素辺の重なり有無（0:なし、1:有り）
        'i1･･･第1の辺の一端の節点番号
        'i2･･･第1の辺の他端の節点番号
        'j1･･･第2の辺の一端の節点番号
        'j2･･･第2の辺の他端の節点番号
        ic = 0
        If i1 = j1 Then
            If i2 = j2 Then
                ic = 1
            End If
        ElseIf i1 = j2 Then
            If i2 = j1 Then
                ic = 1
            End If
        End If

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Tors2DMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            KdataBackup = Val(sr.ReadLine)
            Kscreen = Val(sr.ReadLine)
            KcomSide = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Tors2DMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(KdataBackup)
        sw.WriteLine(Kscreen)
        sw.WriteLine(KcomSide)
        sw.Close()

    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
        ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i) + 0.001
            xydraw(i - 1).Y = Ydraw(i) + 0.001
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub PnumImproveNew(ByRef mb As Integer, ByVal mbx As Integer, _
    ByVal xbuf() As Single, ByVal ybuf() As Single)
        '節点付番の改良
        'mb･･･改良後のﾊﾞﾝﾄﾞ幅
        'mbx･･･改良前のﾊﾞﾝﾄﾞ幅
        'xbuf(),ybuf()･･･節点の座標
        Dim i, ie, k, kx, mbb As Integer
        Dim pa, qa, ratio As Single
        Dim kcbuf(Npoint) As Integer

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie
        ratio = 1.2
        i = 0
        Call AxisCheck2D(i, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ

        For k = 1 To 2
            Call ShellSort2(k, kcbuf, xbuf, ybuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            If k = 1 Then
                kx = 1
                mbb = mb
            ElseIf mb < mbb Then
                kx = k
                mbb = mb
            End If
        Next k

        If mbb < mbx Then
            Call ShellSort2(kx, kcbuf, xbuf, ybuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            For i = 1 To Npoint
                Kcomp(i) = kcbuf(i)
            Next i
        Else
            mb = mbx
        End If

    End Sub

    Sub RoundPrep(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim i, ib As Integer
        Dim b2 As Single

        b2 = zb
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub SfileInput(ByRef fxx$)
        '断面生成ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'fxx$･･･ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, ii, j, k As Integer
        Dim b1, b2 As Single
        aa$ = ""
        ii = FreeFile()
        FileOpen(ii, fxx$, OpenMode.Input)
        Input(ii, Title$)
        Input(ii, aa$) : Input(ii, Npoint)
        Input(ii, aa$) : Input(ii, Nelem)

        ReDim NNelem(Nelem, 8), X(Npoint), Y(Npoint)

        Input(ii, aa$)
        For i = 1 To Nelem
            Input(ii, k)
            For j = 1 To 8
                Input(ii, NNelem(i, j))
            Next j
        Next i

        Input(ii, aa$)
        For i = 1 To Npoint
            Input(ii, k)
            Input(ii, b1)
            Input(ii, b2)
            X(i) = b1
            Y(i) = b2
        Next i

        FileClose(ii)

    End Sub

    Sub SfileOut(ByRef fxx$)
        '断面生成ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'fxx$･･･ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, ii, j As Integer
        ii = FreeFile()
        FileOpen(ii, fxx$, OpenMode.Output)
        WriteLine(ii, Title$)
        WriteLine(ii, "節点数", Npoint)
        WriteLine(ii, "要素数", Nelem)

        WriteLine(ii, "要素データ (要素番号、8節点番号)")
        For i = 1 To Nelem
            aa$ = Str$(i)
            For j = 1 To 8
                aa$ = aa$ + ",   " + Str$(NNelem(i, j))
            Next j
            PrintLine(ii, aa$)
        Next i

        WriteLine(ii, "節点データ（節点番号、Ｘ座標、Ｙ座標）")
        For i = 1 To Npoint
            aa$ = Str$(i) + ",   " + Str$(X(i)) + ",   " + Str$(Y(i))
            PrintLine(ii, aa$)
        Next i
        FileClose(ii)

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub ShellSort2(ByVal kind As Integer, ByRef kcbuf() As Integer, _
ByVal xbuf() As Single, ByVal ybuf() As Single)
        'ｼｪﾙｿｰﾄ
        'kind･･･主ｿｰﾄの座標軸番号（1:X軸、2:Y軸）
        'kcbuf()･･･元の節点の変数順序
        'xbuf(),ybuf()･･･節点の座標
        Dim i As Integer
        Dim nsort(Npoint) As Integer
        Dim work(Npoint) As Single

        Select Case kind
            Case 1
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = xbuf(i)
                Next i
            Case 2
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = ybuf(i)
                Next i
        End Select

        Call ShellSortStd(Npoint, nsort, work)    'ｼｪｯﾙｿｰﾄ基本

        Call MiniSort2(kind, nsort, work, xbuf, ybuf)    'ﾐﾆｿｰﾄ

        For i = 1 To Npoint
            kcbuf(nsort(i)) = i
        Next i

    End Sub

    Sub ShellSortStd(ByVal nx As Integer, ByRef nsort() As Integer, ByRef work() As Single)
        'ｼｪﾙｿｰﾄ基本
        'nx･･･変数の数
        'nsort()･･ｿｰﾄ順の元変数の番号
        'work()･･･ｿｰﾄ順の変数値
        Dim i, ib, j, m As Integer
        Dim varb As Single

        m = nx
        Do While m > 1
            m = m \ 2
            For j = m + 1 To nx
                varb = work(j)
                ib = nsort(j)
                i = j - m
                Do While work(i) >= varb
                    work(i + m) = work(i)
                    nsort(i + m) = nsort(i)
                    i = i - m
                    If i <= 0 Then
                        Exit Do
                    End If
                Loop
                work(i + m) = varb
                nsort(i + m) = ib
            Next j
        Loop

    End Sub

    Sub TimCal(ByVal ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Torsion2D.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Torsion2D.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub UFileAddB()
        'UFileBの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileB(i) = UFileB(0) Then
                For j = i - 1 To 0 Step -1
                    UFileB(j + 1) = UFileB(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileB(i + 1) = UFileB(i)
            Next i
        End If
        UseFileBOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileBIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Tors2UFileB.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileB(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileBOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\Tors2UFileB.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileB(i) <> "" Then
                sw.WriteLine(UFileB(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub XYCalc2(ByRef icheck As Integer, ByRef xb As Single, _
        ByRef yb As Single)
        'X,Yの作画座標計算
        'icheck･･･作画時のXY入れ替えの有無（0:なし、1:有り）
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            If icheck = 0 Then
                xb = b1 * XYscale + Xlower
                yb = Yupper - b2 * XYscale * Cfix
            Else
                xb = b2 * XYscale + Xlower
                yb = b1 * XYscale * Cfix + Ylower
            End If
        Else
            If icheck = 0 Then
                xb = b1 * XYscale / Cfix + Xlower
                yb = Yupper - b2 * XYscale
            Else
                xb = b2 * XYscale / Cfix + Xlower
                yb = b1 * XYscale + Ylower
            End If
        End If
    End Sub

End Module
